@@ -1,4 +1,4 @@
-# Copyright (c) 2021 ING Wholesale Banking Advanced Analytics
+# Copyright (c) 2022 ING Wholesale Banking Advanced Analytics
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy of
 # this software and associated documentation files (the "Software"), to deal in
@@ -26,18 +26,21 @@ from typing import List
 import numpy as np
 import pandas as pd
 import pybase64
-from ing_theme_matplotlib import mpl_style
 from matplotlib import pyplot as plt
 
+import popmon.config
 from popmon.resources import templates_env
 
 NUM_NS_DAY = 24 * 3600 * int(1e9)
 
 logger = logging.getLogger()
-mpl_style(dark=False)
+if popmon.config.themed:
+    from ing_theme_matplotlib import mpl_style
+
+    mpl_style(dark=False)
 
 
-def plt_to_str(format="png"):
+def plt_to_str(fig, format="png"):
     """Outputting plot as a base64 encoded string or as svg image.
 
     :return: base64 encoded plot image or svg image
@@ -47,15 +50,15 @@ def plt_to_str(format="png"):
     if format == "png":
         tmpfile = BytesIO()
 
-        plt.savefig(tmpfile, format="png")
-        plt.close()
+        fig.savefig(tmpfile, format="png")
+        plt.close(fig)
 
         return pybase64.b64encode(tmpfile.getvalue()).decode("utf-8")
     elif format == "svg":
         tmpfile = StringIO()
 
-        plt.savefig(tmpfile, format="svg")
-        plt.close()
+        fig.savefig(tmpfile, format="svg")
+        plt.close(fig)
 
         return tmpfile.getvalue().encode("utf-8")
     else:
@@ -122,22 +125,25 @@ def plot_bars_b64(data, labels=None, bounds=None, ylim=False, skip_empty=True):
             y_max += spread
             y_min -= spread
 
+            yellow = (1.0, 200 / 255, 0.0)
+            red = (1.0, 0.0, 0.0)
+
             if not isinstance(max_r, (list, tuple)):
-                ax.axhline(y=max_r, xmin=0, xmax=1, color="r")
+                ax.axhline(y=max_r, xmin=0, xmax=1, color=red)
             else:
-                ax.plot(index, max_r, color="r")
+                ax.plot(index, max_r, color=red)
             if not isinstance(max_r, (list, tuple)):
-                ax.axhline(y=max_y, xmin=0, xmax=1, color="y")
+                ax.axhline(y=max_y, xmin=0, xmax=1, color=yellow)
             else:
-                ax.plot(index, max_y, color="y")
+                ax.plot(index, max_y, color=yellow)
             if not isinstance(max_r, (list, tuple)):
-                ax.axhline(y=min_y, xmin=0, xmax=1, color="y")
+                ax.axhline(y=min_y, xmin=0, xmax=1, color=yellow)
             else:
-                ax.plot(index, min_y, color="y")
+                ax.plot(index, min_y, color=yellow)
             if not isinstance(max_r, (list, tuple)):
-                ax.axhline(y=min_r, xmin=0, xmax=1, color="r")
+                ax.axhline(y=min_r, xmin=0, xmax=1, color=red)
             else:
-                ax.plot(index, min_r, color="r")
+                ax.plot(index, min_r, color=red)
             if y_max > y_min:
                 ax.set_ylim(y_min, y_max)
         elif ylim:
@@ -152,7 +158,7 @@ def plot_bars_b64(data, labels=None, bounds=None, ylim=False, skip_empty=True):
     ax.grid(True, linestyle=":")
 
     fig.tight_layout()
-    return plt_to_str()
+    return plt_to_str(fig)
 
 
 def render_traffic_lights_table(feature, data, metrics: List[str], labels: List[str]):
@@ -185,13 +191,14 @@ def render_alert_aggregate_table(feature, data, metrics: List[str], labels: List
         for c2, label in enumerate(labels):
             a = data[c1][c2] / row_max if row_max and row_max != 0 else 0
             if metric.endswith("green"):
-                rgba = (0, 128, 0, a)
+                background_rgba = (0, 128, 0, a)
             elif metric.endswith("yellow"):
-                rgba = (255, 255, 0, a)
+                background_rgba = (255, 200, 0, a)
             else:
-                rgba = (255, 0, 0, a)
-            rgba = (str(v) for v in rgba)
-            colors[metric][label] = (rgba, data[c1][c2])
+                background_rgba = (255, 0, 0, a)
+            background_rgba = (str(v) for v in background_rgba)
+            text_color = "white" if a > 0.5 else "black"
+            colors[metric][label] = (text_color, background_rgba, data[c1][c2])
 
     return templates_env(
         "table.html",
@@ -280,7 +287,7 @@ def plot_traffic_lights_b64(data, labels=None, skip_empty=True):
 
     fig.tight_layout()
 
-    return plt_to_str()
+    return plt_to_str(fig)
 
 
 def grouped_bar_chart_b64(data, labels, legend):
@@ -320,7 +327,7 @@ def grouped_bar_chart_b64(data, labels, legend):
 
     fig.tight_layout()
 
-    return plt_to_str()
+    return plt_to_str(fig)
 
 
 def plot_overlay_1d_histogram_b64(
@@ -361,7 +368,7 @@ def plot_overlay_1d_histogram_b64(
         if len(hists) != len(hist_names):
             raise ValueError("length of hist and hist_names are different")
 
-    plt.figure(figsize=(9, 7))
+    fig, ax = plt.subplots(figsize=(9, 7))
 
     alpha = 1.0 / len(hists)
     for i, hist in enumerate(hists):
@@ -407,7 +414,7 @@ def plot_overlay_1d_histogram_b64(
                 width = np.diff(bin_edges)
 
             # plot histogram
-            plt.bar(
+            ax.bar(
                 bin_edges[:-1],
                 bin_values,
                 width=width,
@@ -417,10 +424,10 @@ def plot_overlay_1d_histogram_b64(
 
             # set x-axis properties
             if xlim:
-                plt.xlim(xlim)
+                ax.set_xlim(xlim)
             else:
-                plt.xlim(min(bin_edges), max(bin_edges))
-            plt.xticks(fontsize=12, rotation=90 if is_ts else 0)
+                ax.set_xlim(min(bin_edges), max(bin_edges))
+            ax.tick_params(axis="x", labelsize=12, labelrotation=90 if is_ts else 0)
 
         # plot categories
         else:
@@ -434,7 +441,7 @@ def plot_overlay_1d_histogram_b64(
 
             # plot histogram
             tick_pos = np.arange(len(labels)) + 0.5
-            plt.bar(tick_pos, values, width=0.8, alpha=alpha, label=hist_names[i])
+            ax.bar(tick_pos, values, width=0.8, alpha=alpha, label=hist_names[i])
 
             # set x-axis properties
             def xtick(lab):
@@ -444,19 +451,120 @@ def plot_overlay_1d_histogram_b64(
                     lab = lab[:17] + "..."
                 return lab
 
-            plt.xlim((0.0, float(len(labels))))
-            plt.xticks(
-                tick_pos, [xtick(lab) for lab in labels], fontsize=12, rotation=90
-            )
+            ax.set_xlim((0.0, float(len(labels))))
+            ax.set_xticks(tick_pos)
+            ax.set_xticklabels([xtick(lab) for lab in labels], fontsize=12, rotation=90)
 
     # set common histogram properties
-    plt.xlabel(x_label, fontsize=14)
-    plt.ylabel(str(y_label) if y_label is not None else "Bin count", fontsize=14)
-    plt.yticks(fontsize=12)
-    plt.grid()
-    plt.legend()
+    ax.set_xlabel(x_label, fontsize=14)
+    ax.set_ylabel(str(y_label) if y_label is not None else "Bin count", fontsize=14)
+    ax.tick_params(axis="y", labelsize=12)
+    ax.grid()
+    ax.legend()
 
-    return plt_to_str()
+    return plt_to_str(fig)
+
+
+def plot_heatmap_b64(
+    hist_values,
+    hist_bins,
+    date,
+    x_label,
+    hist_name,
+    y_label=None,
+    is_num=False,
+    is_ts=False,
+    cmap="autumn_r",
+    top=20,
+    xlim=None,
+):
+    """Create and plot heatmap of column values.
+
+    Copyright Eskapade:
+    Kindly taken from Eskapade package and then modified. Reference link:
+    https://github.com/KaveIO/Eskapade/blob/master/python/eskapade/visualization/vis_utils.py#L397
+    License: https://github.com/KaveIO/Eskapade-Core/blob/master/LICENSE
+    Modifications copyright ING WBAA.
+
+    :param list hist_values: values of heatmap in a 2d numpy array =
+    :param list hist_bins: bin labels/edges on y-axis
+    :param list date: dates for x/time axis of heatmap
+    :param str x_label: Label for heatmap x-axis
+    :param list hist_names: list of histogram names. default is [].
+    :param str y_label: Label for histogram y-axis. default is None.
+    :param bool is_num: True if observable to plot is numeric. default is True.
+    :param bool is_ts: True if observable to plot is a timestamp. default is False.
+    :param int top: only print the top 20 characters of x-labels and y-labels. default is 20.
+    :param float cmap: the colormap for heeatmap. default is autumn_r.
+    :param tuple xlim: set the x limits of the current axes. default is None.
+    :return: base64 encoded plot image
+    :rtype: str
+    """
+    if hist_name:
+        if len(hist_name) == 0:
+            raise ValueError("length of heatmap names is zero")
+
+    fig = plt.figure(figsize=(40, 20))
+
+    assert hist_values is not None and len(
+        hist_values
+    ), "Heatmap bin values have not been set."
+    assert hist_bins is not None and len(hist_bins), "Heatmap binning has not been set."
+
+    # basic attribute check: time stamps treated as numeric.
+    if is_ts:
+        is_num = True
+
+    # plot numeric and time stamps
+    if is_num:
+        bin_edges = hist_bins
+        bin_values = hist_values
+        assert (
+            len(bin_edges) == len(bin_values) + 1
+        ), "bin edges (+ upper edge) and bin values have inconsistent lengths: {:d} vs {:d}. {}".format(
+            len(bin_edges), len(bin_values), x_label
+        )
+        return ""
+
+    # plot categories
+    else:
+        labels = hist_bins
+        values = hist_values
+        assert len(labels) == len(
+            values
+        ), "labels and values have different array lengths: {:d} vs {:d}. {}".format(
+            len(labels), len(values), x_label
+        )
+
+        # plot histogram
+        tick_pos_x = np.arange(len(date))
+        tick_pos_y = np.arange(len(labels))
+        plt.imshow(values, cmap)
+
+        # set x-axis properties
+        def xtick(lab):
+            """Get x-tick."""
+            lab = str(lab)
+            if len(lab) > top:
+                lab = lab[:17] + "..."
+            return lab
+
+        # plt.xlim((0.0, float(len(date))))
+        plt.xticks(tick_pos_x, date, fontsize=20, rotation=90)
+        plt.yticks(tick_pos_y, [xtick(lab) for lab in labels], fontsize=20)
+        im_ratio = values.shape[0] / values.shape[1]
+
+    # set common histogram properties
+
+    # Plot vertical colorbar
+    cbar = plt.colorbar(fraction=0.047 * im_ratio)
+    cbar.ax.tick_params(labelsize=20)
+
+    plt.xlabel("Time Bins", fontsize=20)
+    plt.ylabel(x_label, fontsize=20)
+    plt.grid()
+
+    return {"name": hist_name, "plot": plt_to_str(fig)}
 
 
 def _prune(values, last_n=0, skip_first_n=0, skip_last_n=0):
